/* SAMHAIN file system integrity testing                                   */
/* Copyright (C) 2000 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */

#include "config_xor.h"

#include "samhain.h"
#include "sh_error.h"

#undef  FIL__
#define FIL__  _("sh_err_console.c")

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>

extern int  OnlyStderr;

 
#if !defined(O_NONBLOCK)
#if defined(O_NDELAY)
#define O_NONBLOCK  O_NDELAY
#else
#define O_NONBLOCK  0
#endif
#endif


/* ---- Print out a message. ----
 */
int  sh_log_console (char *errmsg)
{
  int    fd[2] = { -1, -1};
  FILE * ofile = NULL;
  int    sflags;
  int    cc;
  int    ccMax = 1;
  int    retval = -1;

  int    error;
  struct sigaction sa_new, sa_old;

  SL_ENTER(_("sh_log_console"));

  ASSERT_RET((errmsg != NULL), _("errmsg != NULL"), 0)

  sa_new.sa_handler = SIG_IGN;
  sigemptyset(&sa_new.sa_mask);
  sa_new.sa_flags   = 0;

  /* Ignore SIGPIPE in case the console is actually a pipe.
   */
  retry_sigaction(SIGPIPE, &sa_new, &sa_old);
 
  if (sh.flag.isdaemon == S_FALSE)
    {
      fprintf (stderr, "%s\n", errmsg);
      retry_sigaction(SIGPIPE, &sa_old, NULL);
      SL_RETURN(0, _("sh_log_console"));
    }

  /* --- daemon && initialized ---
   */
  if ( (OnlyStderr == S_FALSE) ) 
    {
      fd[0] = open ( sh.srvcons.name, O_WRONLY|O_APPEND|O_NOCTTY|O_NONBLOCK);
      if (fd[0] >= 0) {
	sflags = retry_fcntl(fd[0], F_GETFL, 0);
	retry_fcntl(fd[0], F_SETFL, sflags & ~O_NONBLOCK);
      }

      if (sh.srvcons.alt != NULL && sh.srvcons.alt[0] != '\0')
	{
	  fd[1] = open (sh.srvcons.alt, O_WRONLY|O_APPEND|O_NOCTTY|O_NONBLOCK);
	  if (fd[1] >= 0) {
	    sflags = retry_fcntl(fd[1], F_GETFL, 0);
	    retry_fcntl(fd[1], F_SETFL, sflags & ~O_NONBLOCK);
	    ccMax = 2;
	  }
	}

      for (cc = 0; cc < ccMax; ++cc)
	{
      
	  if (fd[cc] < 0)
	    {
	      error = errno;
	      sh_error_handle ((-1), FIL__, __LINE__, error, MSG_SRV_FAIL,
			       _("console"), 
			       (cc == 0) ? sh.srvcons.name : sh.srvcons.alt);
	      /*
	      retry_sigaction(SIGPIPE, &sa_old, NULL);
	      SL_RETURN(-1, _("sh_log_console"));
	      */
	    }

	  if (fd[cc] >= 0)
	    {
	      ofile = fdopen (fd[cc], "a");

	      if (ofile != NULL)
		{
		  (void) fprintf (ofile, _("%s\r\n"), errmsg);
		  (void) fclose  (ofile);
		  retval = 0;
#if 0
		  {
		    FILE * dfile = fopen("/dev/console", "w");
		    fprintf (dfile, "Interval %ld\n", sh.fileCheck.alarm_interval);
		    fclose (dfile);
		  }
#endif

		}
	      else
		{
		  error = errno;
		  (void) close (fd[cc]);
		  sh_error_handle((-1), FIL__, __LINE__, error, MSG_SRV_FAIL,
				  _("console"), 
				  (cc==0) ? sh.srvcons.name : sh.srvcons.alt);
		  /*
		    retry_sigaction(SIGPIPE, &sa_old, NULL);
		    SL_RETURN(-1, _("sh_log_console"));
		  */
		}
	    }
	}
    }
  else
    retval = 0;

  retry_sigaction(SIGPIPE, &sa_old, NULL);
  SL_RETURN(retval, _("sh_log_console"));
}


